import { __awaiter } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { Box, Text, Button, Grid, GridItem, Image, Input, Menu, MenuButton, MenuList, Radio, Stack, useDisclosure, useRadio, useRadioGroup, Flex, useOutsideClick, Portal, } from "@chakra-ui/react";
import { ChevronDown } from "@feedloop/icon";
import React from "react";
import { defineComponent, useSourceHooks, useEvaluate, ListItemVariables, RuntimeContext, useRuntimeInput, useDisableCondition, } from "@qorebase/app-component";
import { fallbackImg } from "@qorebase/app-runtime/src/utils/fallbackImage";
import InputComponentLabel from "@qorebase/app-runtime/src/components/InputComponentLabel";
import { FixedSizeList } from "react-window";
import AutoSizer from "react-virtualized-auto-sizer";
import InfiniteLoader from "react-window-infinite-loader";
import { usePageParam } from "@qorebase/app-runtime/src/hooks/usePageParam";
const Option = (props) => {
    const value = useEvaluate(props.properties.itemValue ||
        String(props.item.id) ||
        String(props.source.itemId));
    const label = useEvaluate(props.properties.itemLabel);
    if (String(props.dataValue) === String(value))
        props.setSelected(label);
    return (_jsx(Flex, Object.assign({ style: Object.assign({}, props.style), onClick: () => {
            props.onClose();
            props.onChange(value);
        }, _hover: {
            bg: "gray.300",
        }, backgroundColor: String(props.dataValue) === String(value) && "gray.100", paddingX: "0.8rem", cursor: "pointer", alignItems: "center" }, { children: label })));
};
const RadioItem = React.memo(function RadioItem(props) {
    const value = useEvaluate(props.properties.itemValue ||
        String(props.item.id) ||
        String(props.source.itemId));
    const { getInputProps, htmlProps, state, getCheckboxProps, getLabelProps } = useRadio(props.getRadioProps({ value }));
    const image = useEvaluate(props.properties.itemImage);
    const label = useEvaluate(props.properties.itemLabel);
    return (_jsxs(GridItem, Object.assign({ bg: "surface.900", rounded: "md", border: "1px", borderColor: state.isChecked ? props.app.theme.options.primaryColor : "gray.200", style: {
            border: "1px solid rgb(226, 232, 240)",
            pointerEvents: props.isDisabled ? "none" : "auto",
            background: props.isDisabled && "#E7E7E7",
        }, _hover: !state.isChecked && {
            bg: "gray.200",
            borderColor: "gray.300",
        } }, { children: [props.properties.style === "text_choice" && (_jsx(Radio, Object.assign({ w: "full", p: "1em", color: props.properties.style === "text_choice" && "gray.300", colorScheme: props.properties.style === "text_choice"
                    ? `var(--chakra-colors-qore-primary);`
                    : undefined }, props.getRadioProps({ value }), { children: _jsx(Text, Object.assign({ color: props.isDisabled && "#A2A2A2" }, { children: label })) }))), props.properties.style === "image_choice" && (_jsxs(Box, Object.assign({ as: "label", cursor: "pointer" }, htmlProps, { children: [_jsx("input", Object.assign({}, getInputProps({}), { hidden: true })), _jsxs(Stack, Object.assign({ direction: props.properties.layout === "horizontal_component"
                            ? "row"
                            : "column", alignItems: "center", justifyContent: props.properties.layout === "vertical_component" && "center", spacing: "1em", w: "full", p: "1em", _hover: !state.isChecked &&
                            state.isHovered && { border: "1px solid gray.300" }, border: "1px solid transparent", borderColor: state.isChecked && `var(--chakra-colors-qore-primary);`, borderRadius: "md" }, getCheckboxProps(), { children: [props.properties.style === "image_choice" && (_jsx(Image, Object.assign({ src: image, alt: label, fallbackSrc: fallbackImg, w: "64px", h: "64px", borderRadius: "md", style: {
                                    border: "1px solid rgb(226, 232, 240)",
                                    filter: props.isDisabled ? "grayscale(1)" : "grayscale(0)",
                                    WebkitFilter: props.isDisabled
                                        ? "grayscale(1)"
                                        : "grayscale(0)",
                                    opacity: props.isDisabled ? "0.5" : "1",
                                } }, getLabelProps()))), _jsx(Text, Object.assign({ display: props.properties.hideLabel && "none", color: props.isDisabled && "#A2A2A2" }, { children: label }))] }))] })))] })));
});
const CustomRadioGroup = React.memo(function CustomRadioGroup(props) {
    var _a;
    const { getRootProps, getRadioProps } = useRadioGroup({
        value: props.variableData.value,
        onChange: props.variableData.onChange,
    });
    return (_jsx(Grid, Object.assign({ templateColumns: props.isVerticalAlign
            ? "repeat(1,1fr)"
            : {
                base: "repeat(1,1fr)",
                md: "repeat(2,1fr)",
                lg: "repeat(3,1fr)",
            }, gap: 2 }, getRootProps(), { children: (_a = props.listData) === null || _a === void 0 ? void 0 : _a.map((item) => (_jsx(ListItemVariables, Object.assign({ variables: item }, { children: _jsx(RadioItem, Object.assign({ item: item, dataValue: props.variableData.value, getRadioProps: getRadioProps, isDisabled: props.isDisabled }, props)) }), item.id))) })));
});
const DropdownListComponent = defineComponent({
    name: "Dropdown List",
    type: "list",
    icon: "SelectInput",
    group: "input",
    display: "block",
    defaultProps: {
        label: "Dropdown List",
        placeholder: "Placeholder here",
        data: {
            inputID: "DropdownList",
            field: { type: "relation", options: {} },
            validation: {
                required: false,
            },
        },
        style: "dropdown",
        itemLabel: "",
        itemImage: "",
        itemValue: "",
        verticalAlignment: false,
        hideLabel: false,
        layout: "vertical_component",
    },
    propDefinition: {
        label: { group: "Design", type: "expression", options: {} },
        placeholder: {
            group: "Design",
            type: "expression",
            options: {},
        },
        data: {
            group: "Design",
            type: "state",
            options: {
                inputID: "DropdownList",
                field: { type: "relation", options: {} },
            },
        },
        style: {
            group: "Design",
            type: "string",
            options: {
                format: "select",
                options: [
                    {
                        label: "Dropdown",
                        value: "dropdown",
                    },
                    {
                        label: "Text Choice",
                        value: "text_choice",
                    },
                    {
                        label: "Image Choice",
                        value: "image_choice",
                    },
                ],
            },
        },
        itemLabel: {
            group: "Design",
            type: "expression",
            options: {},
            section: "dropdown",
        },
        itemImage: {
            group: "Design",
            type: "string",
            options: {
                format: "image",
            },
            section: "dropdown",
            hideOrShownOn: {
                type: "shownOn",
                expression: {
                    type: "formula",
                    formula: `style == "image_choice"`,
                },
            },
        },
        itemValue: {
            group: "Design",
            type: "expression",
            options: {},
            section: "dropdown",
        },
        verticalAlignment: {
            group: "Design",
            type: "boolean",
            options: {},
            section: "dropdown",
            hideOrShownOn: {
                type: "hideOn",
                expression: {
                    type: "formula",
                    formula: `style == "dropdown"`,
                },
            },
        },
        hideLabel: {
            group: "Design",
            type: "boolean",
            options: {},
            section: "dropdown",
            hideOrShownOn: {
                type: "shownOn",
                expression: {
                    type: "formula",
                    formula: `style == "image_choice"`,
                },
            },
        },
        layout: {
            group: "Design",
            section: "dropdown",
            hideOrShownOn: {
                type: "shownOn",
                expression: {
                    type: "formula",
                    formula: `style == "image_choice"`,
                },
            },
            type: "string",
            options: {
                format: "button",
                options: [
                    {
                        icon: "IconTopLeft",
                        value: "vertical_component",
                    },
                    {
                        icon: "IconLeft",
                        value: "horizontal_component",
                    },
                ],
            },
        },
    },
    Editor: () => __awaiter(void 0, void 0, void 0, function* () {
        return {
            data: (props) => {
                const pageParam = usePageParam();
                const currentPage = RuntimeContext.useSelectState((state) => state.maps.page[pageParam]);
                const schema = RuntimeContext.useSelectState((state) => state.schema);
                const tablesAndViews = React.useMemo(() => [].concat(schema.tables, schema.views, schema.insights), [schema]);
                const pageView = React.useMemo(() => tablesAndViews.find((view) => view.name === (currentPage === null || currentPage === void 0 ? void 0 : currentPage.sourceID)), [currentPage === null || currentPage === void 0 ? void 0 : currentPage.sourceID]);
                return (_jsx(props.PropertyEditor.state, { propDef: props.propDef, propName: props.propName, value: props.value, onChange: props.onChange, view: pageView }));
            },
        };
    }),
    Component: (props) => {
        var _a, _b;
        const { isOpen, onClose, onToggle } = useDisclosure();
        const boxWidth = React.useRef(null);
        const [selected, setSelected] = React.useState("");
        const [search, setSearch] = React.useState("");
        const [pageLoader, setPageLoader] = React.useState(0);
        const sourceHooks = useSourceHooks(Object.assign(Object.assign({}, props.source), { offset: 50 * pageLoader }));
        const [currentData, setCurrentData] = React.useState([]);
        const { data: listData = [], status } = sourceHooks.useFetchList();
        const title = useEvaluate(props.properties.label);
        const placeholder = useEvaluate(props.properties.placeholder);
        const wrapperRef = React.useRef(null);
        const [searching, setSearching] = React.useState(false);
        useOutsideClick({
            enabled: searching ? false : true,
            ref: wrapperRef,
            handler: () => {
                onClose();
            },
        });
        const data = useRuntimeInput(props.properties.data, props.data.page.row, {
            validate: (value) => {
                if (typeof value === "undefined") {
                    return { error: "" };
                }
                else if (!value) {
                    return { error: "Required field, Please enter the data" };
                }
            },
        });
        const validateData = (currentData, newData) => {
            if (!currentData.length) {
                return newData;
            }
            else {
                const filteredData = [...currentData];
                newData.forEach((newItem) => {
                    const exist = currentData.find((item) => item.id === newItem.id);
                    if (!exist) {
                        filteredData.push(newItem);
                    }
                });
                return filteredData;
            }
        };
        const getData = React.useCallback(() => {
            return validateData(currentData, listData);
        }, [currentData, listData]);
        const isVerticalAlign = React.useMemo(() => props.properties.verticalAlignment, [props.properties.verticalAlignment]);
        const styleType = React.useMemo(() => props.properties.style, [props.properties.style]);
        React.useEffect(() => {
            if (!!props.source.target && status === "success" && !!listData.length) {
                if (pageLoader === 0) {
                    setCurrentData(listData);
                }
                else {
                    setCurrentData(getData());
                }
            }
        }, [pageLoader, status, props.source.target, listData]);
        React.useEffect(() => {
            setCurrentData(listData);
            setPageLoader(0);
        }, [props.source.target, props.source.params]);
        // @ts-ignore
        const { isDisabled } = useDisableCondition(props.instance);
        const disabledProperties = {
            border: "1px solid rgb(226, 232, 240)",
            pointerEvents: isDisabled ? "none" : "auto",
            background: isDisabled && "#E7E7E7",
            color: isDisabled && "#A2A2A2",
        };
        const identifier = React.useMemo(() => {
            var _a;
            if (search !== "") {
                return (_a = `${props.properties.itemLabel}`.match(/[^\b(item)[.]\w+/g)) === null || _a === void 0 ? void 0 : _a[0];
            }
            return null;
        }, [search]);
        const filteredData = React.useMemo(() => {
            return currentData.filter((item) => {
                var _a;
                if (search === "") {
                    return item;
                }
                else if ((_a = item[identifier]) === null || _a === void 0 ? void 0 : _a.toLowerCase().includes(search.toLocaleLowerCase())) {
                    return item;
                }
            });
        }, [currentData, search, identifier]);
        const isItemLoaded = React.useCallback((index) => {
            if (index === currentData.length - 1 && listData.length) {
                return false;
            }
            return true;
        }, [currentData, listData]);
        const loadMoreItems = React.useCallback(() => {
            return new Promise((resolve) => {
                setPageLoader((prev) => prev + 1);
                resolve();
            });
        }, []);
        const handleLoadMore = React.useCallback(() => {
            if (status === "success") {
                setPageLoader((prev) => prev + 1);
            }
        }, [status]);
        const renderRow = React.useCallback(({ index, style }) => {
            if (isItemLoaded(index) && listData) {
                return (_jsx(ListItemVariables, Object.assign({ variables: filteredData[index] }, { children: _jsx(Option, Object.assign({ item: filteredData[index], onChange: data.onChange, dataValue: data.value, setSelected: setSelected }, props, { "data-test": "dropdown-list-option", search: search, style: style, onClose: onClose })) }), filteredData[index].id));
            }
            else {
                return (_jsx(Flex, Object.assign({ paddingX: "0.8rem", alignItems: "center", height: "34px", style: style }, { children: "Loading..." })));
            }
        }, [filteredData, data, setSelected]);
        const Options = React.useCallback(() => {
            return (_jsx(AutoSizer, { children: ({ width, height }) => (_jsx(InfiniteLoader, Object.assign({ isItemLoaded: isItemLoaded, itemCount: filteredData.length, loadMoreItems: loadMoreItems }, { children: ({ onItemsRendered, ref }) => (_jsx(FixedSizeList, Object.assign({ height: height, itemCount: filteredData.length, itemSize: 34, width: width, onItemsRendered: onItemsRendered, ref: ref }, { children: renderRow }))) }))) }));
        }, [
            filteredData.length,
            isItemLoaded,
            loadMoreItems,
            props.source.params,
            props.source.target,
        ]);
        return (_jsxs(Box, { children: [_jsx(InputComponentLabel, { title: title, name: "dropdown-list", isRequired: (_a = props.properties.data.validation) === null || _a === void 0 ? void 0 : _a.required, isEmpty: title.length === 0 }), styleType === "dropdown" && (_jsx(Box, Object.assign({ ref: wrapperRef }, { children: _jsxs(Menu, Object.assign({ autoSelect: false, isOpen: isOpen }, { children: [_jsx(MenuButton, Object.assign({ as: Button, style: disabledProperties, bgColor: "surface.900", ref: boxWidth, textAlign: "left", borderRadius: "md", variant: "outline", rightIcon: _jsx(ChevronDown, {}), width: "100%", _focus: { shadow: "0px" }, fontWeight: "normal", "data-test": "dropdown-list-menu-button", onClick: () => {
                                    onToggle();
                                } }, { children: data.value ? selected : placeholder })), _jsx(Portal, { children: _jsxs(MenuList, Object.assign({ border: "0px", width: (_b = boxWidth === null || boxWidth === void 0 ? void 0 : boxWidth.current) === null || _b === void 0 ? void 0 : _b.offsetWidth, zIndex: 3 }, { children: [_jsx(Box, Object.assign({ paddingX: "0.8rem" }, { children: _jsx(Input, { value: search, onChange: (e) => setSearch(e.target.value), placeholder: "Search...", mb: 2, onFocus: () => setSearching(true), onBlur: () => setSearching(false), onTouchStart: () => setSearching(true) }) })), _jsxs(Box, Object.assign({ maxHeight: "11.5rem", overflowY: "auto" }, { children: [_jsx(Text, Object.assign({ color: "gray", py: 2, px: 4, style: { cursor: "default" } }, { children: placeholder })), _jsx("div", Object.assign({ style: {
                                                        width: "100%",
                                                        height: "100px",
                                                        flex: "1 1 auto",
                                                    } }, { children: _jsx(Options, {}) }))] }))] })) })] })) }))), (styleType === "text_choice" || styleType === "image_choice") && (_jsxs(Box, { children: [_jsx(CustomRadioGroup, Object.assign({ variableData: data, listData: currentData, isVerticalAlign: isVerticalAlign, isDisabled: isDisabled }, props)), _jsx(Flex, Object.assign({ mt: 2, justifyContent: "center", w: "full", flexDir: { base: "column", lg: "row" }, display: listData.length && listData.length === 50 ? "flex" : "none" }, { children: _jsx(Button, Object.assign({ variant: "outline", bg: "surface.900", onClick: handleLoadMore }, { children: "Load more" })) }))] })), !!data.validation.error && (_jsx(Text, Object.assign({ fontSize: "sm", color: "red", "data-test": "text-input-error" }, { children: data.validation.error })))] }));
    },
});
export default DropdownListComponent;
//# sourceMappingURL=dropdownList.js.map